import json
from types import SimpleNamespace

class Upstreams(object):
  def __init__(self, client, type):
    self._type = type
    self._client = client

  def getUpstream(self, upstream):
    r = self._client.request('GET',"/api/6/" + self._type + "/upstreams/" + upstream)
    upstreamInfo = json.loads(r.data, object_hook=lambda d: SimpleNamespace(**d))
    if r.status != 200 and r.status != 404:
      raise str(r.status) + ": " + r.reason
    if isinstance(upstreamInfo, SimpleNamespace):
      return upstreamInfo
    return None

  def getUpstreamServer(self, upstream, server):
    r = self._client.request('GET',"/api/6/" + self._type + "/upstreams/" + upstream + "/servers/" + str(server))
    serverInfo = json.loads(r.data, object_hook=lambda d: SimpleNamespace(**d))
    if r.status != 200 and r.status != 404:
      raise str(r.status) + ": " + r.reason
    if isinstance(serverInfo, SimpleNamespace):
      return serverInfo
    return None

  def getUpstreamServers(self, upstream):
    r = self._client.request('GET',"/api/6/" + self._type + "/upstreams/" + upstream + "/servers")
    upstreamInfo = json.loads(r.data, object_hook=lambda d: SimpleNamespace(**d))
    if r.status != 200 and r.status != 404:
      raise str(r.status) + ": " + r.reason
    if isinstance(upstreamInfo, list):
      return upstreamInfo
    return None

  def getAllUpstreams(self):
    r = self._client.request('GET',"/api/6/" + self._type + "/upstreams")
    upstreams = json.loads(r.data, object_hook=lambda d: SimpleNamespace(**d))
    if r.status != 200 and r.status != 404:
      raise str(r.status) + ": " + r.reason
    if isinstance(upstreams, SimpleNamespace):
      return upstreams
    return None

  def addServer(self, upstream, server, **kwargs):
    kwargs['server'] = server
    return self.addServerByDict(upstream, kwargs)

  def addServerByDict(self, upstream, server):
    headers = {
      "Content-Type": "application/json"
    }
    if not isinstance(server, dict):
      raise TypeError('server must be a dict')
    r = self._client.request(
      'POST',"/api/6/" + self._type + "/upstreams/" + upstream + "/servers/",
      body = json.dumps(server),
      headers = headers
    )
    serverInfo = json.loads(r.data, object_hook=lambda d: SimpleNamespace(**d))
    if r.status != 201:
      raise BaseException(str(r.status) + ": " + r.reason + "->" + json.dumps(json.loads(r.data)))
    if isinstance(serverInfo, SimpleNamespace):
      return serverInfo
    return None

  def deleteServer(self, upstream, server):
    r = self._client.request('DELETE',"/api/6/" + self._type + "/upstreams/" + upstream + "/servers/" + str(server))
    serverInfo = json.loads(r.data, object_hook=lambda d: SimpleNamespace(**d))
    if r.status != 200:
      raise BaseException(str(r.status) + ": " + r.reason + "->" + json.dumps(json.loads(r.data)))
    return serverInfo

  def resetStats(self, upstream):
    r = self._client.request('DELETE',"/api/6/" + self._type + "/upstreams/" + upstream)
    if r.status != 204:
      raise BaseException(str(r.status) + ": " + r.reason)
    return True

  def setServerDown(self, upstream, server):
    return self.setServerFlag(upstream, server, 'down', 'true')
  
  def setServerUp(self, upstream, server):
    return self.setServerFlag(upstream, server, 'down', 'false')

  def setServerDrain(self, upstream, server):
    return self.setServerFlag(upstream, server, 'drain', 'true')

  def setServerFlag(self, upstream, server, stateFlag, state):
    serverState = {
      'id': str(server),
      stateFlag: state
    }

    return self.setServerFlagByDict(upstream, serverState)

  def setServerFlagByDict(self, upstream, server):
    headers = {
      "Content-Type": "application/json"
    }
    serverID = server['id']
    del server['id']
    if not isinstance(server, dict):
      raise TypeError('server must be a dict')
    r = self._client.request(
      'PATCH',"/api/6/" + self._type + "/upstreams/" + upstream + "/servers/" + serverID,
      body = json.dumps(server),
      headers = headers
    )
    serverInfo = json.loads(r.data, object_hook=lambda d: SimpleNamespace(**d))
    if r.status != 200:
      raise BaseException(str(r.status) + ": " + r.reason + "->" + json.dumps(json.loads(r.data)))
    if isinstance(serverInfo, SimpleNamespace):
      return serverInfo
    return None


import json

class KeyValueStore(dict):
  def __init__(self, client, zone, values=None):
    self._zone = zone
    self._client = client
    self._initValues = values
    self._cache = {}
    if isinstance(values,dict):
      for key in values:
        self.set(key, values[key])
    elif values is not None:
      raise TypeError("Incompatible type for values. Must be dict or None. Class type "+ str(values.__class__) + " used")

  def set(self, key, value):
    method = 'POST'
    v = self.get(key)
    if v == value:
      print('New value and old value are the same')
      return
    if v:
      method = 'PATCH'
    body = {
      key: value
    }
    r = self._client.request(method, "/api/6/http/keyvals/" + self._zone, body=json.dumps(body))
    if r.status != 201 and r.status != 204:
      raise KeyError("Could not store key value. " + str(r.status) + ": " + r.reason)

  def get(self, key):
    r = self._client.request('GET',"/api/6/http/keyvals/" + self._zone)
    self._cache = json.loads(r.data)
    if r.status != 200 and r.status != 404:
      raise str(r.status) + ": " + r.reason
    if key in self._cache:
      return self._cache[key]
    return None

  def getAll(self):
    r = self._client.request('GET',"/api/6/http/keyvals/" + self._zone)
    self._cache = json.loads(r.data)
    if r.status != 200 and r.status != 404:
      raise str(r.status) + ": " + r.reason
    return self._cache

  def deleteAll(self):
    r = self._client.request('delete',"/api/6/http/keyvals/" + self._zone)
    if r.status != 204:
      raise BaseException(str(r.status) + ": " + r.reason)
    return

  def items(self):
    return self.getAll().keys()
  
  def keys(self):
    return self.getAll().keys()

  def __setitem__(self, key, value):
    if isinstance(key, str):
      self.set(key,value)
    return
    
  def __getitem__(self, key):
    if isinstance(key, str):
      return self.get(key)
    
  def __iter__(self):
    return self.getAll().__iter__()

  def __repr__(self):
    return self.getAll().__repr__()
  
  def __str__(self):
    return self.getAll().__str__()

  def __len__(self):
    return self.getAll().__len__()

  def __delitem__(self, key):
    if isinstance(key, str):
      return self.set(key,None)
    raise TypeError("key object must be string")

  def __contains__(self, key):
    if isinstance(key, str):
      return key in self.getAll()
    raise TypeError("key object must be string")

  def __del__(self):
    self.deleteAll()
    return


import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="nginxPlusAPI", # Replace with your own username
    version="0.0.2",
    author="James Jones",
    author_email="jam.jones@f5.com",
    description="Python API Client for NGINX Plus API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/nginx-architects/python-nginxPlusAPI",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: Freely Distributable",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


import urllib3
import multiprocessing
from nginxPlusAPI import KeyValueStore
from nginxPlusAPI import Upstreams

class Client(object):

  def __init__(self, host, poolSize=multiprocessing.cpu_count()):
    super(Client, self).__init__()
    self.host = host
    self._poolSize = poolSize
    self._HTTPPool = urllib3.connectionpool.connection_from_url(host, maxsize=poolSize)
    self.HTTPUpstreams = Upstreams(self, 'http')
    self.StreamUpstreams = Upstreams(self, 'stream')
    self._KeyValueStoreFactory = KeyValueStore

  def KeyValueStore(self, zone, values=None):
    return self._KeyValueStoreFactory(self, zone, values)
  
  def request(self, *args, **kwargs):
    return self._HTTPPool.request(*args, **kwargs)